#ifndef MAXENT_CPP_H
#define MAXENT_CPP_H

#include <iostream>

class moments_cpp;
class multipliers_cpp;
class maxent_cpp;

std::ostream & operator << (std::ostream &, const moments_cpp &);
std::ostream & operator << (std::ostream &, const multipliers_cpp &);

std::istream & operator >> (std::istream &, moments_cpp &);
std::istream & operator >> (std::istream &, multipliers_cpp &);

class moments_cpp
{

 public:

    moments_cpp(const int d = 0, const int p = 0);

    moments_cpp(const moments_cpp &);

    ~moments_cpp();

    // Operators

    double & operator () (const int *) const;

    moments_cpp & operator = (const moments_cpp &);
    // Equals

    moments_cpp & set_dim_pow (const int d, const int p);

    int get_dim () const;

    int get_pow () const;

    int get_size () const;

    const moments_cpp & print(std::ostream & stream) const;

    const moments_cpp & print() const;

    friend std::ostream & operator << (std::ostream &, const moments_cpp &);

    friend std::istream & operator >> (std::istream &, moments_cpp &);

    friend class maxent_cpp;

 private:

    void * pt;

};

class multipliers_cpp
{

 public:

    multipliers_cpp();

    multipliers_cpp(const multipliers_cpp &);

    ~multipliers_cpp();

    // Operators

    double & operator () (const int *) const;

    double operator () (const double *) const;
    // Compute P(x), the length of x should be dim

    multipliers_cpp & operator = (const multipliers_cpp &);

    // Equals

    int get_dim () const;

    int get_pow () const;

    int get_size () const;

    const multipliers_cpp & print(std::ostream & stream) const;

    const multipliers_cpp & print() const;

    friend std::ostream & operator << (std::ostream &, const multipliers_cpp &);

    friend std::istream & operator >> (std::istream &, multipliers_cpp &);

    friend class maxent_cpp;

 private:

    void * pt;

};


class maxent_cpp
{

 public:

    maxent_cpp(const int s = 0); // Constructor
    ~maxent_cpp();


    maxent_cpp & set_quad_size (const int s = 0);

    maxent_cpp & set_tol(const double tol = 0);

    int operator () (const moments_cpp & c, multipliers_cpp & m);
    // Returns error exit status
    // Values are: 0 if no error has occured; 1 if error in Newton
    // method (NaN); 2 if error in line search (NaN or delta too small)

    maxent_cpp & info(const bool i); // set true for stdin info output

 private:

    void * pt;

};

#endif /* MAXENT_CPP_H */
