#include <stdio.h>
#include <math.h>
#include <maxent.h>

int main()
{

    int i;

    /* Initialize 50-point Gauss-Hermite quadrature */
    maxent mxnt = maxent_set_quad_size(50);

    /* Initialize multipliers */
    multipliers p = multiplier_init();

    /* Initialize moments of dimension 1 and power up to 4 */
    moments m = moment_set_dim_pow(1,4);

    /* Set moment constraint values, powers 0 through 4 */
    i=0; moment_set_value(m,&i,1.0);
    i=1; moment_set_value(m,&i,1.0);
    i=2; moment_set_value(m,&i,1.5);
    i=3; moment_set_value(m,&i,2.4);
    i=4; moment_set_value(m,&i,4.0);

    /* Check the values of moments */
    puts("------------- Moments -------------");
    moment_print(m,stdout);

    /* Computing the constraints */
    i = maxent_compute(mxnt,m,p);

    /* Check if error has occured */
    if(i) puts("Error has occured");
    else  puts("No error has occured");

    /* Check the values of computed multipliers*/
    puts("------------- Multipliers -------------");
    multiplier_print(p,stdout);

    /* Retrieve values and print them */

    puts("Check multipliers manually");
    for(i=0;i<=multiplier_get_pow(p);++i)
      printf(" %g",multiplier_get_value(p,&i));
    puts("");

    /* Here is another way to get the shape of the PDF. If one
     * provides a floating-point vector x to p, it will return the
     * value of the polynomial consisting of Lagrange multipliers and
     * powers at x. One can do it repeatedly for multiple
     * coordinates. Below is an example of how to retrieve values of
     * the computed PDF on a 81-point 1D-domain for -3<x<3. */

    int Nx=81;
    double limx = 3.0;
    double x;
    double scale[Nx];
    double PDF_values[Nx];

    double deltax = 2.0*limx/(Nx-1);

    for(i=0; i<Nx; ++i){
      x = -limx + deltax*i;
      scale[i] = x;
      PDF_values[i] = exp(multiplier_get_poly_value(p,&x));
      /* PDF is an exponent of the polynomial */
    }

    /* Now the values of the maxent PDF are written into the PDF_values
     * array. Here is an example of how to output then in the gnuplot
     * format */

    FILE * output = fopen("maxent_example_1D_c.dat","wb");

    for (i=0; i<Nx; ++i)
      fprintf(output,"%g\t%g\n",scale[i],PDF_values[i]);

    fclose(output);

    maxent_free(mxnt);
    multiplier_free(p);
    moment_free(m);

    /* Typing 'plot "maxent_example_1D_c.dat" with lines' at the
     * gnuplot prompt will produce the line plot with this example
     * PDF. */

    return 0;

}

