#include <iostream>
#include <fstream>
#include <cmath>

#include <maxent.hh>

using namespace std;

int main()
{

    int i;
    maxent compute_maxent(50); // 50-point Gauss-Hermite quadrature
    multipliers p; // This is where the multipliers will be written
    moments m(1,4); // One-dimensional 4-moment problem

    // Specify moments here
    i=0; m(&i)=1; // PDF norm should be 1
    i=1; m(&i)=1; // First moment (mean) 1
    i=2; m(&i)=1.5; // Second moment (variance) 1.5
    i=3; m(&i)=2.4; // Third moment 2.4
    i=4; m(&i)=4.0; // Fourth moment 4.0

    // Check what was actually written into m
    cout << "------------- Contents of m -------------" << endl;
    m.print();

    i = compute_maxent(m,p); // Compute the problem

    if(i) cout << "Error has occured" << endl;
    else cout << "No error has occured" << endl;

    // Check what is actually computed

    cout << "------------- Contents of p -------------" << endl;
    p.print();

    // Retrieve multipliers and print them

    cout << "Check multipliers manually" << endl;
    for(i=0;i<=p.get_pow();++i)
      cout << " " << p(&i);
    cout << endl;

    // Here is another way to get the shape of the PDF. If one
    // provides a floating-point vector x to p, it will return the
    // value of the polynomial consisting of Lagrange multipliers and
    // powers at x. One can do it repeatedly for multiple
    // coordinates. Below is an example of how to retrieve values of
    // the computed PDF on a 81-point 1D-domain for -3<x<3.

    int Nx=81;
    double limx = 3.0;
    double x;
    double scale[Nx];
    double PDF_values[Nx];

    double deltax = 2.0*limx/(Nx-1);

    for(i=0; i<Nx; ++i){
      x = -limx + deltax*i;
      scale[i] = x;
      PDF_values[i] = exp(p(&x));
      // PDF is an exponent of the polynomial
    }

    // Now the values of the maxent PDF are written into the PDF_values
    // array. Here is an example of how to output then in the gnuplot
    // format

    ofstream output("maxent_example_1D_cpp.dat",ios::out);

    for (i=0; i<Nx; ++i)
      output << scale[i] << "\t" << PDF_values[i] << endl;

    // Typing 'plot "maxent_example_1D_cpp.dat" with lines' at the
    // gnuplot prompt will produce the line plot with this example
    // PDF.

    return 0;
}

