      PROGRAM MAIN
      IMPLICIT NONE

C     Declare variables for moments, multipliers and maxent
      INTEGER*8 MOM, MULT, MAXENT
      INTEGER I
      DOUBLE PRECISION X, TEMP

      DOUBLE PRECISION LIMX
      DOUBLE PRECISION SCALE(61)
      DOUBLE PRECISION PDF_VALUES(61)
      DOUBLE PRECISION DELTAX

C     Initialize moments to 1 dimension and power up to 4
      CALL MOMENT_SET_DIM_POW(MOM,1,4)

C     Initialize multipliers
      CALL MULTIPLIER_INIT(MULT)

C     Initialize maxent with 50-point Gauss-Hermite quadrature
      CALL MAXENT_SET_QUAD_SIZE(MAXENT,50)

C     Set moment values for power 0 through 4
      I=0
      X=1.0
      CALL MOMENT_SET_VALUE(MOM,I,X)

      I=1
      X=1.0
      CALL MOMENT_SET_VALUE(MOM,I,X)

      I=2
      X=1.5
      CALL MOMENT_SET_VALUE(MOM,I,X)

      I=3
      X=2.4
      CALL MOMENT_SET_VALUE(MOM,I,X)

      I=4
      X=4.0
      CALL MOMENT_SET_VALUE(MOM,I,X)

C     Check values of moments
      CALL MOMENT_PRINT(MOM)

C     Compute maxent
      CALL MAXENT_COMPUTE(MAXENT,MOM,MULT,I)

C     Check if error has occured
      IF (I .GT. 0) THEN
         WRITE (*,*) 'Error in maxent'
      ELSE
         WRITE (*,*) 'No error in maxent'
      ENDIF
      
C     Check values of multipliers
      CALL MULTIPLIER_PRINT(MULT)

C     Retrieve multipliers manually and print them
      I=0
      DO WHILE (I .le. 4)
         CALL MULTIPLIER_GET_VALUE(MULT,I,X)
         WRITE (*,*) X
         I=I+1
      ENDDO

C     Here is another way to get the shape of the PDF. If one provides a
C     floating-point vector X to MULT, it will return the value of the
C     polynomial consisting of Lagrange multipliers and powers at X. One
C     can do it repeatedly for multiple coordinates. Below is an example
C     of how to retrieve values of the computed PDF on a 61 points
C     1D-domain for -3<x<3.

      LIMX = 3.0

      DELTAX = 2.0*LIMX/60

      I = 1
      DO WHILE (I .LE. 61)
         X = -LIMX + DELTAX*(I-1)
         SCALE(I) = X
         CALL MULTIPLIER_GET_POLY_VALUE(MULT,X,TEMP)
         PDF_VALUES(I) = DEXP(TEMP)
C     PDF is an exponent of the polynomial
         I = I+1
      ENDDO

C     Now the values of the maxent PDF are written into the PDF_values
C     array. Here is an example of how to output then in the gnuplot
C     format

      OPEN(UNIT=6,FILE='maxent_example_1D_f.dat')
      I = 1
      DO WHILE (I .LE. 61)
         WRITE (6,*) SCALE(I),PDF_VALUES(I)
         I = I+1
      ENDDO
      CLOSE(6)

C     Typing 'plot "maxent_example_1D_f.dat" with lines' at the gnuplot
C     prompt will produce the line plot with this example PDF.

      CALL MOMENT_FREE(MOM)
      CALL MULTIPLIER_FREE(MULT)
      CALL MAXENT_FREE(MAXENT)

      END PROGRAM MAIN
