	PROGRAM MAIN
	IMPLICIT NONE

	INTEGER*8 MOM, MULT, MAXENT
	INTEGER I(2), STATUS
	DOUBLE PRECISION X(2), TEMP


	INTEGER IX,IY
	DOUBLE PRECISION LIMX, LIMY, MULT21
	DOUBLE PRECISION SCALE(61)
	DOUBLE PRECISION PDF_VALUES(61,61)
	DOUBLE PRECISION DELTAX, DELTAY

C       First initialize a set of 2-dimensional moment constraints of up
C       to 4th order. Altogether we will have 15 constraints.

	CALL MOMENT_SET_DIM_POW(MOM,2,4)

C       Second, initialize a set of multipliers.

	CALL MULTIPLIER_INIT(MULT)

C       Third, initialize the maximum entropy routine object. In this
C       example it will have 50-point Gauss-Hermite quadrature.

	CALL MAXENT_SET_QUAD_SIZE(MAXENT,50)

C       Now define 16 moment constraints for the problem.
C       In order to set the value for each moment in the moments class,
C       powers of all variables corresponding to the current moment have
C       to be provided, plus the value of the moment itself. In order to
C       provide powers in our 2D setting, we have just reserved I. I(1)
C       will provide the power of the first variable, and I(2) will
C       provide the power of the second variable.

C       Zero power constraint, supposed to be 1 for a PDF.
	I(1)=0
	I(2)=0
	TEMP = 1.0
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)

C       Set up first powers (which are the mean states)

C       Mean of the first variable
	I(1)=1
	I(2)=0
	TEMP = 0.1
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)

C       Mean of the second variable
	I(1)=0
	I(2)=1
	TEMP = 0.2
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)

C       Set up second power moments (covariances), there are three of them

C       Variance of the first variable (x^2)
	I(1)=2
	I(2)=0
	TEMP = 0.9
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)

C       Covariance between the two variables (xy)
	I(1)=1
	I(2)=1
	TEMP = 0.2
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)

C       Variance of the first variable (y^2)
	I(1)=0
	I(2)=2
	TEMP = 1.1
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)


C       Set up third power moments (skewness), there are four of them

C       Skewness of the first variable (x^3)
	I(1)=3
	I(2)=0
	TEMP = 0.2
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)

C       Cross-skewness between the two variables (x^2y)
	I(1)=2
	I(2)=1
	TEMP = -0.05
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)

C       Another cross-skewness between the two variables (xy^2)
	I(1)=1
	I(2)=2
	TEMP = 0.1
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)

C       Skewness of the second variable (y^3)
	I(1)=0
	I(2)=3
	TEMP = -0.3
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)


C       Set up fourth power moments (flatness), there are five of them

C       Flatness of the first variable (x^4)
	I(1)=4
	I(2)=0
	TEMP = 1.8
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)

C       Cross-flatness between the two variables (x^3y)
	I(1)=3
	I(2)=1
	TEMP = 0.03
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)

C       Another cross-flatness between the two variables (x^2y^2)
	I(1)=2
	I(2)=2
	TEMP = 0.8
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)

C       Another cross-flatness between the two variables (xy^3)
	I(1)=1
	I(2)=3
	TEMP = 0.04
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)

C       Flatness of the second variable (y^4)
	I(1)=0
	I(2)=4
	TEMP = 2.0
	CALL MOMENT_SET_VALUE(MOM,I,TEMP)
  
C       Checking contents of MOM

	WRITE (*,*) '------------- Contents of MOM -------------'
	CALL MOMENT_PRINT(MOM)

	CALL MAXENT_COMPUTE(MAXENT,MOM,MULT,STATUS)

C       Status is nonzero if errors have been encountered (i.e. the
C       algorithm did not converge)

	IF (STATUS .GT. 0) THEN
	   WRITE (*,*) 'Error in maxent'
	ELSE
	   WRITE (*,*) 'No error in maxent'
	ENDIF

C       MULT now contains the values of multipliers, if no error has
C       happened.

C       Checking contents of MULT

	WRITE (*,*) '------------- Contents of MULT -------------'
	CALL MULTIPLIER_PRINT(MULT)

C       The Lagrange multipliers of MULT are obtained in a similar
C       manner as the constraints are set for MOM. Namely:

	I(1)=2
	I(2)=1
	CALL MULTIPLIER_GET_VALUE(MULT,I,MULT21)

C       MULT21 now has the value of the Lagrange multiplier which
C       corresponds to the x^2y power. One can retrieve all the values
C       of the Lagrange multipliers in the same manner.

	WRITE (*,*) 'The Lagrange multiplier for x^2y is ', MULT21

C       Here is another way to get the shape of the PDF. If one provides
C       a floating-point vector X to MULT, it will return the value of
C       the polynomial consisting of Lagrange multipliers and powers at
C       X. One can do it repeatedly for multiple coordinates. Below is
C       an example of how to retrieve values of the computed PDF on a
C       61x61 points 2D-domain for -3<x<3 and -3<y<3.

	LIMX = 3.0
	LIMY = 3.0;

	DELTAX = 2.0*LIMX/60
	DELTAY = 2.0*LIMY/60

	IY = 1
	DO WHILE (IY .LE. 61)
	   X(2) = -LIMY + DELTAY*(IY-1)
	   SCALE(IY) = X(2)
C       Since the scales in x and y are identical, using the same for
C       both
	   IX = 1
	   DO WHILE (IX .LE. 61)
	      X(1) = -LIMX + DELTAX*(IX-1)
	      CALL MULTIPLIER_GET_POLY_VALUE(MULT,X,MULT21)
	      PDF_VALUES(IX,IY) = DEXP(MULT21)
C       PDF is an exponent of the polynomial
	      IX = IX+1
	   ENDDO
	   IY = IY+1
	ENDDO

C       Now the values of the maxent PDF are written into the PDF_values
C       array. Here is an example of how to output then in the gnuplot
C       format

	OPEN(UNIT=6,FILE='maxent_example_2D_f.dat')
	IX = 1
	DO WHILE (IX .LE. 61)
	   IY = 1
	   DO WHILE (IY .LE. 61)
	      WRITE (6,*) SCALE(IX),SCALE(IY),PDF_VALUES(IX,IY)
	      IY = IY+1
	   ENDDO
	   WRITE (6,*) ''
	   IX = IX+1
	ENDDO
	CLOSE(6)
	
C       Typing 'splot "maxent_example_2D_f.dat" with lines' at the gnuplot
C       prompt will produce the 3D mesh plot with this example PDF.

	CALL MAXENT_FREE(MAXENT)
	CALL MULTIPLIER_FREE(MULT)
	CALL MOMENT_FREE(MOM)
  
	END PROGRAM MAIN
